==> approval-1.0.18.tgz: package/src/reducers/index.js <==

const defaultState = [
	{  id: 0, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Лермонтов М.Ю.'  }},
	{  id: 1, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Толстой Л.Н.'    }},
	{  id: 2, itemType: 'stage',     data: { prevItemID:   0                 }},
	{  id: 3, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Есенин С.А.'     }},
	{  id: 4, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Чехов А.П.'      }},
	{  id: 5, itemType: 'stage',     data: { prevItemID:   4                 }},
	{  id: 6, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Достоевский Ф.М.'}},
	{  id: 7, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Маяковский В.В.' }},
	{  id: 8, itemType: 'stage',     data: { prevItemID:   7                 }},
	{  id: 9, itemType: 'approver',  data: { employeeCompany: 'ООО \"Амадеус-трейдинг\"', employeePosition: 'Заместитель начальника департаммента', employeeName: 'Тургенев И.С.'   }}
]

// const removeApprover = (state, action, stageIndex) => {
// 	if(stageIndex !== action.stage) {
// 		return state;
// 	}

// 	return state.filter((approver, index) => index !== action.index )
// }

// const addApprover = (state, action) => {
// 	return [...state.slice(0, action.index), 
// 			{ approver: action.approver },
// 			...state.slice(action.index)]
// }

// const changeState = (state, action) => {
// 	return [...state.slice(0, action.stage), 
// 			approvalStage(state[action.stage], action),
// 			...state.slice(action.stage + 1)]
// }

// const approvalStage = (state, action, stageIndex) => {
// 	switch (action.type) {
// 		case 'ADD_APPROVER':
// 			return [...state, { approver: action.approver }]
// 		case 'REMOVE_APPROVER':
// 			return removeApprover(state, action, stageIndex)
// 		case 'ADD_TO':
// 			return addApprover(state, action)

// 		case 'MOVE_APPROVER': 
// 			return removeApprover(state, action, stageIndex)

// 		case 'SPLIT_GROUP':
// 			if (state.length > 1 && action.index !== state.length - 1) { //разбиваем, если в этапе больше одного сотрудника и сотрудник не последний в списке
// 				return [
// 					[...state.slice(0, action.index + 1)], 
// 					[...state.slice(action.index + 1)]
// 				]
// 			}
// 			return [state]
// 		// case 'REMOVE_APPROVER':
// 		// 	if (id !== action.index) {
// 		// 		return state; //если индекс этапа не совпадает с индексом нужного нам этапа, то запишем его в массив неизмененным
// 		// 	}
// 		// 	return [...state.slice(0,state.indexOf()), ...state.slice(state.indexOf()+1)]

// 			// Object.assign(
// 			// 	{},
// 			// 	state, //весь state оставляем нетронутым, кроме типа маршрута
// 			// 	{ parallel: !state.parallel } //изменяем тип маршрута на противоположный
// 			// );
// 		// case 'TOGGLE_ROUTE':
// 		// 	if (stageIndex !== action.index) {
// 		// 		return state; //если индекс этапа не совпадает с индексом нужного нам этапа, то запишем его в массив неизмененным
// 		// 	}
// 		// 	return Object.assign(
// 		// 		{},
// 		// 		state, //весь state оставляем нетронутым, кроме типа маршрута
// 		// 		{ parallel: !state.parallel } //изменяем тип маршрута на противоположный
// 		// 	);
// 		default:
// 			return state; //желательно всегда возвращать текущее состояние, чтобы избежать лишних багов
// 	}
// }




const routeItem = (state, action) => {
	switch (action.type) {
		case 'ADD_ROUTE_ITEM':
			return { id: action.id, itemType: action.itemType, data: action.data }
		// case 'REMOVE_APPROVER':
		// 	state.id === action.id ? 
		default:
			return state;
	}	
}

const approvalRoute = (state = defaultState, action) => {
	switch (action.type) {
		case 'ADD_ROUTE_ITEM':
			if(action.itemType == 'approver') {
				return [
					...state, //предыдущие этапы оставляем нетронутыми
					routeItem(undefined, action) //добавляем сотрудника к последнему этапу согласования 
				]				
			} else if(action.itemType == 'stage') {
				console.log(action)
				let placeIndex = state.findIndex(item => item.id === action.data.prevItemID) + 1
				console.log('placeIndex :'+ placeIndex)
				return [
					...state.slice(0, placeIndex), routeItem(undefined, action), //предыдущие этапы оставляем нетронутыми
					...state.slice(placeIndex)//добавляем сотрудника к последнему этапу согласования 
				]								
			} else {
				return state
			}
		case 'REMOVE_ROUTE_ITEM': {
			let tempState = state.filter(item => item.id !== action.id), //удаляем нужный item
				itemsCount = tempState.length
				
			if (itemsCount <= 1) {
				return tempState
			} else { //нужно убрать лишниe stage (если остался сверху или снизу или два подряд)
				return tempState.filter((item, index) => !((index === 0 || index === itemsCount - 1 || tempState[index-1].itemType === 'stage') && item.itemType === 'stage') )
			}
		}
		// case 'MOVE_APPROVER': {
		// 	// случай, когда последний этап согласования и в нем один человек
		// 	if(state[action.stage].length === 1 && (!state[action.stage + 1] && action.to === 'down' || !state[action.stage - 1] && action.to === 'up')) {
		// 		return state
		// 	}

		// 	let movedApprover = state.filter( (stage, stageIndex) => stageIndex === action.stage )[0]
		// 							.filter( (approver, approverIndex) => approverIndex === action.index )[0] 

		// 	let newState = state.map((stage, index) => approvalStage(stage, action, index))

		// 	if (action.to === 'down') {
		// 		if (newState[action.stage + 1]) {
		// 			// expect(console.log(newState))
		// 			// expect(console.log( newState[action.stage + 1] ))
		// 			return [...newState.slice(0, action.stage + 1), 
		// 				[...[movedApprover], ...newState[action.stage + 1]], 
		// 				...newState.slice(action.stage + 2)].filter(stage => stage.length > 0)
		// 		} else {
		// 			return [...newState.slice(0, action.stage + 1), [movedApprover]].filter(stage => stage.length > 0)
		// 		}				
		// 	} else {
		// 		if (newState[action.stage - 1]) {
		// 			// expect(console.log(newState))
		// 			// expect(console.log( newState[action.stage + 1] ))
		// 			return [...newState.slice(0, action.stage - 1), 
		// 				[...newState[action.stage - 1], ...[movedApprover]], 
		// 				...newState.slice(action.stage)].filter(stage => stage.length > 0)
		// 		} else {
		// 			// expect(console.log(newState))
		// 			return [[movedApprover], ...newState.slice(action.stage)].filter(stage => stage.length > 0)
		// 		}	
		// 	}
		// }
		// case 'ADD_TO':
		// 	return changeState(state, action)
		case 'MOVE_DOWN_STAGE': {
			// if(state[action.stage + 1]) {
			let stageIndex = state.findIndex(item => item.id === action.id),
				prevStageIndex = state.indexOf(state.slice(0, stageIndex).reverse().find(item => item.itemType === 'stage')),
				nextStageIndex = state.indexOf(state.slice(stageIndex + 1).find(item => item.itemType === 'stage'))

				if (prevStageIndex !== -1 && nextStageIndex !== -1) { //перестановка двух средних элементов

					return [
						...state.slice(0, prevStageIndex + 1),
						...state.slice(stageIndex + 1, nextStageIndex), 
						state[stageIndex],
						...state.slice(prevStageIndex + 1, stageIndex),
						...state.slice(nextStageIndex)
					]
				} else if (nextStageIndex !== -1) { //перестановка двух первых элементов
					return [
						...state.slice(stageIndex + 1, nextStageIndex), 
						state[stageIndex],
						...state.slice(0, stageIndex),
						...state.slice(nextStageIndex)
					]
				} else if (prevStageIndex !== -1) { //перестановка двух последних элементов
					return [
						...state.slice(0, prevStageIndex + 1),
						...state.slice(stageIndex + 1), 
						state[stageIndex],
						...state.slice(prevStageIndex + 1, stageIndex) 
					]
				} else { //перестановка двух элементов, когда один этап
					return [
						...state.slice(stageIndex + 1), 
						state[stageIndex],
						...state.slice(0, stageIndex) 
					]
				}
			}
		default:
			return state
	}
}

export default approvalRoute

==> approval-1.0.22.tgz: package/src/reducers/index.js <==
var shortid = require('shortid');

const defaultState = []

// const removeApprover = (state, action, stageIndex) => {
// 	if(stageIndex !== action.stage) {
// 		return state;
// 	}

// 	return state.filter((approver, index) => index !== action.index )
// }

// const addApprover = (state, action) => {
// 	return [...state.slice(0, action.index), 
// 			{ approver: action.approver },
// 			...state.slice(action.index)]
// }

// const changeState = (state, action) => {
// 	return [...state.slice(0, action.stage), 
// 			approvalStage(state[action.stage], action),
// 			...state.slice(action.stage + 1)]
// }

// const approvalStage = (state, action, stageIndex) => {
// 	switch (action.type) {
// 		case 'ADD_APPROVER':
// 			return [...state, { approver: action.approver }]
// 		case 'REMOVE_APPROVER':
// 			return removeApprover(state, action, stageIndex)
// 		case 'ADD_TO':
// 			return addApprover(state, action)

// 		case 'MOVE_APPROVER': 
// 			return removeApprover(state, action, stageIndex)

// 		case 'SPLIT_GROUP':
// 			if (state.length > 1 && action.index !== state.length - 1) { //разбиваем, если в этапе больше одного сотрудника и сотрудник не последний в списке
// 				return [
// 					[...state.slice(0, action.index + 1)], 
// 					[...state.slice(action.index + 1)]
// 				]
// 			}
// 			return [state]
// 		// case 'REMOVE_APPROVER':
// 		// 	if (id !== action.index) {
// 		// 		return state; //если индекс этапа не совпадает с индексом нужного нам этапа, то запишем его в массив неизмененным
// 		// 	}
// 		// 	return [...state.slice(0,state.indexOf()), ...state.slice(state.indexOf()+1)]

// 			// Object.assign(
// 			// 	{},
// 			// 	state, //весь state оставляем нетронутым, кроме типа маршрута
// 			// 	{ parallel: !state.parallel } //изменяем тип маршрута на противоположный
// 			// );
// 		// case 'TOGGLE_ROUTE':
// 		// 	if (stageIndex !== action.index) {
// 		// 		return state; //если индекс этапа не совпадает с индексом нужного нам этапа, то запишем его в массив неизмененным
// 		// 	}
// 		// 	return Object.assign(
// 		// 		{},
// 		// 		state, //весь state оставляем нетронутым, кроме типа маршрута
// 		// 		{ parallel: !state.parallel } //изменяем тип маршрута на противоположный
// 		// 	);
// 		default:
// 			return state; //желательно всегда возвращать текущее состояние, чтобы избежать лишних багов
// 	}
// }




const routeItem = (state, action) => {
	switch (action.type) {
		case 'SET_INITIAL_STATE':
			return Object.assign({}, state, {id: shortid.generate()});
		case 'ADD_ROUTE_ITEM':
			return { id: shortid.generate(), itemType: action.itemType, data: action.data }
		// case 'REMOVE_APPROVER':
		// 	state.id === action.id ? 
		default:
			return state;
	}	
}

const approvalRoute = (state = defaultState, action) => {
	switch (action.type) {
		case 'SET_INITIAL_STATE':
			return [...action.initialState.map(item => routeItem(item, action))]
		case 'ADD_ROUTE_ITEM':
			if(action.itemType == 'approver') {
				return [
					...state, //предыдущие этапы оставляем нетронутыми
					routeItem(undefined, action) //добавляем сотрудника к последнему этапу согласования 
				]				
			} else if(action.itemType == 'stage') {
				let placeIndex = state.findIndex(item => item.id === action.data.prevItemID) + 1
				return [
					...state.slice(0, placeIndex), routeItem(undefined, action), //предыдущие этапы оставляем нетронутыми
					...state.slice(placeIndex)//добавляем сотрудника к последнему этапу согласования 
				]								
			} else {
				return state
			}
		case 'REMOVE_ROUTE_ITEM': {
			let tempState = state.filter(item => item.id !== action.id), //удаляем нужный item
				itemsCount = tempState.length
				
			if (itemsCount <= 1) {
				return tempState
			} else { //нужно убрать лишниe stage (если остался сверху или снизу или два подряд)
				return tempState.filter((item, index) => !((index === 0 || index === itemsCount - 1 || tempState[index-1].itemType === 'stage') && item.itemType === 'stage') )
			}
		}
		// case 'MOVE_APPROVER': {
		// 	// случай, когда последний этап согласования и в нем один человек
		// 	if(state[action.stage].length === 1 && (!state[action.stage + 1] && action.to === 'down' || !state[action.stage - 1] && action.to === 'up')) {
		// 		return state
		// 	}

		// 	let movedApprover = state.filter( (stage, stageIndex) => stageIndex === action.stage )[0]
		// 							.filter( (approver, approverIndex) => approverIndex === action.index )[0] 

		// 	let newState = state.map((stage, index) => approvalStage(stage, action, index))

		// 	if (action.to === 'down') {
		// 		if (newState[action.stage + 1]) {
		// 			// expect(console.log(newState))
		// 			// expect(console.log( newState[action.stage + 1] ))
		// 			return [...newState.slice(0, action.stage + 1), 
		// 				[...[movedApprover], ...newState[action.stage + 1]], 
		// 				...newState.slice(action.stage + 2)].filter(stage => stage.length > 0)
		// 		} else {
		// 			return [...newState.slice(0, action.stage + 1), [movedApprover]].filter(stage => stage.length > 0)
		// 		}				
		// 	} else {
		// 		if (newState[action.stage - 1]) {
		// 			// expect(console.log(newState))
		// 			// expect(console.log( newState[action.stage + 1] ))
		// 			return [...newState.slice(0, action.stage - 1), 
		// 				[...newState[action.stage - 1], ...[movedApprover]], 
		// 				...newState.slice(action.stage)].filter(stage => stage.length > 0)
		// 		} else {
		// 			// expect(console.log(newState))
		// 			return [[movedApprover], ...newState.slice(action.stage)].filter(stage => stage.length > 0)
		// 		}	
		// 	}
		// }
		// case 'ADD_TO':
		// 	return changeState(state, action)
		case 'MOVE_DOWN_STAGE': {
			// if(state[action.stage + 1]) {
			let stageIndex = state.findIndex(item => item.id === action.id),
				prevStageIndex = state.indexOf(state.slice(0, stageIndex).reverse().find(item => item.itemType === 'stage')),
				nextStageIndex = state.indexOf(state.slice(stageIndex + 1).find(item => item.itemType === 'stage'))

				if (prevStageIndex !== -1 && nextStageIndex !== -1) { //перестановка двух средних элементов

					return [
						...state.slice(0, prevStageIndex + 1),
						...state.slice(stageIndex + 1, nextStageIndex), 
						state[stageIndex],
						...state.slice(prevStageIndex + 1, stageIndex),
						...state.slice(nextStageIndex)
					]
				} else if (nextStageIndex !== -1) { //перестановка двух первых элементов
					return [
						...state.slice(stageIndex + 1, nextStageIndex), 
						state[stageIndex],
						...state.slice(0, stageIndex),
						...state.slice(nextStageIndex)
					]
				} else if (prevStageIndex !== -1) { //перестановка двух последних элементов
					return [
						...state.slice(0, prevStageIndex + 1),
						...state.slice(stageIndex + 1), 
						state[stageIndex],
						...state.slice(prevStageIndex + 1, stageIndex) 
					]
				} else { //перестановка двух элементов, когда один этап
					return [
						...state.slice(stageIndex + 1), 
						state[stageIndex],
						...state.slice(0, stageIndex) 
					]
				}
			}
		default:
			return state
	}
}

export default approvalRoute
